@@ -803,7 +803,7 @@ xmlURIUnescapeString(const char *str, int len, char *target) {
     if (str == NULL)
 	return(NULL);
     if (len <= 0) len = strlen(str);
-    if (len <= 0) return(NULL);
+    if (len < 0) return(NULL);
 
     if (target == NULL) {
 	ret = (char *) xmlMalloc(len + 1);
@@ -1966,4 +1966,65 @@ done:
     return(val);
 }
 
+/**
+ * xmlCanonicPath:
+ * @path:  the resource locator in a filesystem notation
+ *
+ * Constructs a canonic path from the specified path. 
+ *
+ * Returns a new canonic path, or a duplicate of the path parameter if the 
+ * construction fails. The caller is responsible for freeing the memory occupied
+ * by the returned string. If there is insufficient memory available, or the 
+ * argument is NULL, the function returns NULL.
+ */
+#define IS_WINDOWS_PATH(p) 					\
+	((p != NULL) &&						\
+	 (((p[0] >= 'a') && (p[0] <= 'z')) ||			\
+	  ((p[0] >= 'A') && (p[0] <= 'Z'))) &&			\
+	 (p[1] == ':') && ((p[2] == '/') || (p[2] == '\\')))
+xmlChar*
+xmlCanonicPath(const xmlChar *path)
+{
+#if defined(_WIN32) && !defined(__CYGWIN__)    
+    int len = 0;
+    int i = 0;
+    xmlChar *p = NULL;
+#endif
+    xmlChar *ret;
+    xmlURIPtr uri;
+
+    if (path == NULL)
+	return(NULL);
+    if ((uri = xmlParseURI((const char *) path)) != NULL) {
+	xmlFreeURI(uri);
+	return xmlStrdup(path);
+    }
+
+    uri = xmlCreateURI();
+
+#if defined(_WIN32) && !defined(__CYGWIN__)    
+    len = xmlStrlen(path);
+    if ((len > 2) && IS_WINDOWS_PATH(path)) {
+	uri->scheme = xmlStrdup(BAD_CAST "file");
+	uri->path = xmlMalloc(len + 2);
+	uri->path[0] = '/';
+	p = uri->path + 1;
+	strncpy(p, path, len + 1);
+    } else {
+	uri->path = xmlStrdup(path);
+	p = uri->path;
+    }
+    while (*p != '\0') {
+	if (*p == '\\')
+	    *p = '/';
+	p++;
+    }
+#else
+    uri->path = (char *) xmlStrdup((const char *) path);
+#endif
+    
+    ret = xmlSaveUri(uri);
+    xmlFreeURI(uri);
+    return(ret);
+}
 
