@@ -40,6 +40,8 @@
 #define XINCLUDE_PARSE_TEXT (const xmlChar *) "text"
 #define XINCLUDE_PARSE_ENCODING (const xmlChar *) "encoding"
 
+#define XINCLUDE_MAX_DEPTH 40
+
 /* #define DEBUG_XINCLUDE  */
 #ifdef DEBUG_XINCLUDE
 #ifdef LIBXML_DEBUG_ENABLED
@@ -68,6 +70,7 @@ struct _xmlXIncludeRef {
     xmlNodePtr            inc; /* the included copy */
     int                   xml; /* xml or txt */
     int                 count; /* how many refs use that specific doc */
+    xmlXPathObjectPtr    xptr; /* the xpointer if needed */
 };
 
 typedef struct _xmlXIncludeCtxt xmlXIncludeCtxt;
@@ -83,6 +86,13 @@ struct _xmlXIncludeCtxt {
     int                txtMax; /* size of unparsed documents tab */
     xmlNodePtr        *txtTab; /* array of unparsed text nodes */
     xmlURL         *txturlTab; /* array of unparsed txtuments URLs */
+
+    xmlChar *             url; /* the current URL processed */
+    int                 urlNr; /* number of url stacked */
+    int                urlMax; /* size of url stack */
+    xmlChar *         *urlTab; /* url stack */
+
+    int              nbErrors; /* the number of errors detected */
 };
 
 static int
@@ -111,6 +121,8 @@ xmlXIncludeFreeRef(xmlXIncludeRefPtr ref) {
 	xmlFree(ref->URI);
     if (ref->fragment != NULL)
 	xmlFree(ref->fragment);
+    if (ref->xptr != NULL)
+	xmlXPathFreeObject(ref->xptr);
     xmlFree(ref);
 }
 
@@ -152,6 +164,7 @@ xmlXIncludeNewRef(xmlXIncludeCtxtPtr ctxt, const xmlChar *URI,
         if (ctxt->incTab == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "malloc failed !\n");
+	    ctxt->nbErrors++;
 	    xmlXIncludeFreeRef(ret);
 	    return(NULL);
 	}
@@ -197,9 +210,79 @@ xmlXIncludeNewContext(xmlDocPtr doc) {
     ret->incBase = 0;
     ret->incMax = 0;
     ret->incTab = NULL;
+    ret->nbErrors = 0;
     return(ret);
 }
 
+/**
+ * xmlXIncludeURLPush:
+ * @ctxt:  the parser context
+ * @value:  the url
+ *
+ * Pushes a new url on top of the url stack
+ *
+ * Returns -1 in case of error, the index in the stack otherwise
+ */
+static int
+xmlXIncludeURLPush(xmlXIncludeCtxtPtr ctxt,
+	           const xmlChar *value)
+{
+    if (ctxt->urlNr > XINCLUDE_MAX_DEPTH) {
+	xmlGenericError(xmlGenericErrorContext,
+	    "XInclude: detected a recursion in %s\n",
+			value);
+	ctxt->nbErrors++;
+	return(-1);
+    }
+    if (ctxt->urlTab == NULL) {
+	ctxt->urlMax = 4;
+	ctxt->urlNr = 0;
+	ctxt->urlTab = (xmlChar * *) xmlMalloc(
+		        ctxt->urlMax * sizeof(ctxt->urlTab[0]));
+        if (ctxt->urlTab == NULL) {
+            xmlGenericError(xmlGenericErrorContext, "malloc failed !\n");
+            return (-1);
+        }
+    }
+    if (ctxt->urlNr >= ctxt->urlMax) {
+        ctxt->urlMax *= 2;
+        ctxt->urlTab =
+            (xmlChar * *) xmlRealloc(ctxt->urlTab,
+                                      ctxt->urlMax *
+                                      sizeof(ctxt->urlTab[0]));
+        if (ctxt->urlTab == NULL) {
+            xmlGenericError(xmlGenericErrorContext, "realloc failed !\n");
+            return (-1);
+        }
+    }
+    ctxt->url = ctxt->urlTab[ctxt->urlNr] = xmlStrdup(value);
+    return (ctxt->urlNr++);
+}
+
+/**
+ * xmlXIncludeURLPop:
+ * @ctxt: the parser context
+ *
+ * Pops the top url from the url stack
+ */
+static void
+xmlXIncludeURLPop(xmlXIncludeCtxtPtr ctxt)
+{
+    xmlChar * ret;
+
+    if (ctxt->urlNr <= 0)
+        return;
+    ctxt->urlNr--;
+    if (ctxt->urlNr > 0)
+        ctxt->url = ctxt->urlTab[ctxt->urlNr - 1];
+    else
+        ctxt->url = NULL;
+    ret = ctxt->urlTab[ctxt->urlNr];
+    ctxt->urlTab[ctxt->urlNr] = 0;
+    if (ret != NULL)
+	xmlFree(ret);
+}
+
 /**
  * xmlXIncludeFreeContext:
  * @ctxt: the XInclude context
@@ -215,6 +298,10 @@ xmlXIncludeFreeContext(xmlXIncludeCtxtPtr ctxt) {
 #endif
     if (ctxt == NULL)
 	return;
+    while (ctxt->urlNr > 0)
+	xmlXIncludeURLPop(ctxt);
+    if (ctxt->urlTab != NULL)
+	xmlFree(ctxt->urlTab);
     for (i = 0;i < ctxt->incNr;i++) {
 	if (ctxt->incTab[i] != NULL)
 	    xmlXIncludeFreeRef(ctxt->incTab[i]);
@@ -249,7 +336,8 @@ xmlXIncludeAddNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr cur) {
     xmlChar *parse;
     xmlChar *base;
     xmlChar *URI;
-    int xml = 1; /* default Issue 64 */
+    int xml = 1, i; /* default Issue 64 */
+    int local = 0;
 
 
     if (ctxt == NULL)
@@ -268,9 +356,12 @@ xmlXIncludeAddNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr cur) {
 	href = xmlGetProp(cur, XINCLUDE_HREF);
 	if (href == NULL) {
 	    xmlGenericError(xmlGenericErrorContext, "XInclude: no href\n");
+	    ctxt->nbErrors++;
 	    return(-1);
 	}
     }
+    if (href[0] == '#')
+	local = 1;
     parse = xmlGetNsProp(cur, XINCLUDE_NS, XINCLUDE_PARSE);
     if (parse == NULL) {
 	parse = xmlGetProp(cur, XINCLUDE_PARSE);
@@ -284,6 +375,7 @@ xmlXIncludeAddNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr cur) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value %s for %s\n",
 		            parse, XINCLUDE_PARSE);
+	    ctxt->nbErrors++;
 	    if (href != NULL)
 		xmlFree(href);
 	    if (parse != NULL)
@@ -323,6 +415,7 @@ xmlXIncludeAddNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr cur) {
 	xmlFree(base);
     if (URI == NULL) {
 	xmlGenericError(xmlGenericErrorContext, "XInclude: failed build URL\n");
+	ctxt->nbErrors++;
 	return(-1);
     }
 
@@ -333,6 +426,7 @@ xmlXIncludeAddNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr cur) {
     if (uri == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", URI);
+	ctxt->nbErrors++;
 	return(-1);
     }
     if (uri->fragment != NULL) {
@@ -345,11 +439,27 @@ xmlXIncludeAddNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr cur) {
     if (URL == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", URI);
+	ctxt->nbErrors++;
 	if (fragment != NULL)
 	    xmlFree(fragment);
 	return(-1);
     }
 
+    /*
+     * Check the URL against the stack for recursions
+     */
+    if (!local) {
+	for (i = 0;i < ctxt->urlNr;i++) {
+	    if (xmlStrEqual(URL, ctxt->urlTab[i])) {
+		xmlGenericError(xmlGenericErrorContext,
+		    "XInclude: detected a recursion in %s\n",
+				URL);
+		ctxt->nbErrors++;
+		return(-1);
+	    }
+	}
+    }
+
     ref = xmlXIncludeNewRef(ctxt, URL, cur);
     if (ref == NULL) {
 	return(-1);
@@ -376,6 +486,14 @@ xmlXIncludeRecurseDoc(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc,
     xmlXIncludeCtxtPtr newctxt;
     int i;
 
+    /*
+     * Avoid recursion in already substitued resources
+    for (i = 0;i < ctxt->urlNr;i++) {
+	if (xmlStrEqual(doc->URL, ctxt->urlTab[i]))
+	    return;
+    }
+     */
+
 #ifdef DEBUG_XINCLUDE
     xmlGenericError(xmlGenericErrorContext, "Recursing in doc %s\n", doc->URL);
 #endif
@@ -395,9 +513,16 @@ xmlXIncludeRecurseDoc(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc,
         if (newctxt->incTab == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "malloc failed !\n");
+	    ctxt->nbErrors++;
 	    xmlFree(newctxt);
 	    return;
 	}
+	/*
+	 * copy the urlTab
+	 */
+	newctxt->urlMax = ctxt->urlMax;
+	newctxt->urlNr = ctxt->urlNr;
+	newctxt->urlTab = ctxt->urlTab;
 
 	/*
 	 * Inherit the documents already in use by others includes
@@ -413,6 +538,10 @@ xmlXIncludeRecurseDoc(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc,
 	    newctxt->incTab[i]->count--;
 	    newctxt->incTab[i] = NULL;
 	}
+	newctxt->urlMax = 0;
+	newctxt->urlNr = 0;
+	newctxt->urlTab = NULL;
+
 	xmlXIncludeFreeContext(newctxt);
     }
 #ifdef DEBUG_XINCLUDE
@@ -440,6 +569,7 @@ xmlXIncludeAddTxt(xmlXIncludeCtxtPtr ctxt, xmlNodePtr txt, const xmlURL url) {
         if (ctxt->txtTab == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "malloc failed !\n");
+	    ctxt->nbErrors++;
 	    return;
 	}
         ctxt->txturlTab = (xmlURL *) xmlMalloc(ctxt->txtMax *
@@ -447,6 +577,7 @@ xmlXIncludeAddTxt(xmlXIncludeCtxtPtr ctxt, xmlNodePtr txt, const xmlURL url) {
         if (ctxt->txturlTab == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "malloc failed !\n");
+	    ctxt->nbErrors++;
 	    return;
 	}
     }
@@ -457,6 +588,7 @@ xmlXIncludeAddTxt(xmlXIncludeCtxtPtr ctxt, xmlNodePtr txt, const xmlURL url) {
         if (ctxt->txtTab == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "realloc failed !\n");
+	    ctxt->nbErrors++;
 	    return;
 	}
         ctxt->txturlTab = (xmlURL *) xmlRealloc(ctxt->txturlTab,
@@ -464,6 +596,7 @@ xmlXIncludeAddTxt(xmlXIncludeCtxtPtr ctxt, xmlNodePtr txt, const xmlURL url) {
         if (ctxt->txturlTab == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "realloc failed !\n");
+	    ctxt->nbErrors++;
 	    return;
 	}
     }
@@ -757,6 +890,8 @@ xmlXIncludeCopyXPointer(xmlXIncludeCtxtPtr ctxt, xmlDocPtr target,
     xmlNodePtr list = NULL, last = NULL;
     int i;
 
+    if (source == NULL)
+	source = ctxt->doc;
     if ((ctxt == NULL) || (target == NULL) || (source == NULL) ||
 	(obj == NULL))
 	return(NULL);
@@ -781,9 +916,38 @@ xmlXIncludeCopyXPointer(xmlXIncludeCtxtPtr ctxt, xmlDocPtr target,
 #ifdef LIBXML_DOCB_ENABLED
 		    case XML_DOCB_DOCUMENT_NODE:
 #endif
-		    case XML_XINCLUDE_START:
 		    case XML_XINCLUDE_END:
 			break;
+		    case XML_XINCLUDE_START: {
+	                xmlNodePtr tmp, cur = set->nodeTab[i];
+
+			cur = cur->next;
+			while (cur != NULL) {
+			    switch(cur->type) {
+				case XML_TEXT_NODE:
+				case XML_CDATA_SECTION_NODE:
+				case XML_ELEMENT_NODE:
+				case XML_ENTITY_REF_NODE:
+				case XML_ENTITY_NODE:
+				case XML_PI_NODE:
+				case XML_COMMENT_NODE:
+				    tmp = xmlXIncludeCopyNode(ctxt, target,
+							      source, cur);
+				    if (last == NULL) {
+					list = last = tmp;
+				    } else {
+					xmlAddNextSibling(last, tmp);
+					last = tmp;
+				    }
+				    cur = cur->next;
+				    continue;
+				default:
+				    break;
+			    }
+			    break;
+			}
+			continue;
+		    }
 		    case XML_ATTRIBUTE_NODE:
 		    case XML_NAMESPACE_DECL:
 		    case XML_DOCUMENT_TYPE_NODE:
@@ -843,6 +1007,13 @@ xmlXIncludeCopyXPointer(xmlXIncludeCtxtPtr ctxt, xmlDocPtr target,
  *									*
  ************************************************************************/
 
+typedef struct _xmlXIncludeMergeData xmlXIncludeMergeData;
+typedef xmlXIncludeMergeData *xmlXIncludeMergeDataPtr;
+struct _xmlXIncludeMergeData {
+    xmlDocPtr doc;
+    xmlXIncludeCtxtPtr ctxt;
+};
+
 /**
  * xmlXIncludeMergeOneEntity:
  * @ent: the entity
@@ -852,18 +1023,59 @@ xmlXIncludeCopyXPointer(xmlXIncludeCtxtPtr ctxt, xmlDocPtr target,
  * Inplements the merge of one entity
  */
 static void
-xmlXIncludeMergeEntity(xmlEntityPtr ent, xmlDocPtr doc,
+xmlXIncludeMergeEntity(xmlEntityPtr ent, xmlXIncludeMergeDataPtr data,
 	               xmlChar *name ATTRIBUTE_UNUSED) {
-    xmlEntityPtr ret;
+    xmlEntityPtr ret, prev;
+    xmlDocPtr doc;
+    xmlXIncludeCtxtPtr ctxt;
 
-    if ((ent == NULL) || (doc == NULL))
+    if ((ent == NULL) || (data == NULL))
 	return;
+    ctxt = data->ctxt;
+    doc = data->doc;
+    if ((ctxt == NULL) || (doc == NULL))
+	return;
+    switch (ent->etype) {
+        case XML_INTERNAL_PARAMETER_ENTITY:
+        case XML_EXTERNAL_PARAMETER_ENTITY:
+        case XML_INTERNAL_PREDEFINED_ENTITY:
+	    return;
+        case XML_INTERNAL_GENERAL_ENTITY:
+        case XML_EXTERNAL_GENERAL_PARSED_ENTITY:
+        case XML_EXTERNAL_GENERAL_UNPARSED_ENTITY:
+	    break;
+    }
     ret = xmlAddDocEntity(doc, ent->name, ent->etype, ent->ExternalID,
 			  ent->SystemID, ent->content);
     if (ret != NULL) {
 	if (ent->URI != NULL)
 	    ret->URI = xmlStrdup(ent->URI);
+    } else {
+	prev = xmlGetDocEntity(doc, ent->name);
+	if (prev != NULL) {
+	    if (ent->etype != prev->etype)
+		goto error;
+	
+	    if ((ent->SystemID != NULL) && (prev->SystemID != NULL)) {
+		if (!xmlStrEqual(ent->SystemID, prev->SystemID))
+		    goto error;
+	    } else if ((ent->SystemID != NULL) && (prev->SystemID != NULL)) {
+		if (!xmlStrEqual(ent->ExternalID, prev->ExternalID))
+		    goto error;
+	    } else if ((ent->content != NULL) && (prev->content != NULL)) {
+		if (!xmlStrEqual(ent->content, prev->content))
+		    goto error;
+	    } else {
+		goto error;
+	    }
+
+	}
     }
+    return;
+error:
+    xmlGenericError(xmlGenericErrorContext,
+		"XInclude: mismatch in redefinition of entity %s\n", ent->name);
+    ctxt->nbErrors++;
 }
 
 /**
@@ -900,18 +1112,28 @@ xmlXIncludeMergeEntities(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc,
 
     source = from->intSubset;
     if ((source != NULL) && (source->entities != NULL)) {
+	xmlXIncludeMergeData data;
+
+	data.ctxt = ctxt;
+	data.doc = doc;
+
 	xmlHashScan((xmlHashTablePtr) source->entities,
-		    (xmlHashScanner) xmlXIncludeMergeEntity, doc);
+		    (xmlHashScanner) xmlXIncludeMergeEntity, &data);
     }
     source = from->extSubset;
     if ((source != NULL) && (source->entities != NULL)) {
+	xmlXIncludeMergeData data;
+
+	data.ctxt = ctxt;
+	data.doc = doc;
+
 	/*
 	 * don't duplicate existing stuff when external subsets are the same
 	 */
 	if ((!xmlStrEqual(target->ExternalID, source->ExternalID)) &&
 	    (!xmlStrEqual(target->SystemID, source->SystemID))) {
 	    xmlHashScan((xmlHashTablePtr) source->entities,
-			(xmlHashScanner) xmlXIncludeMergeEntity, doc);
+			(xmlHashScanner) xmlXIncludeMergeEntity, &data);
 	}
     }
     return(0);
@@ -945,6 +1167,7 @@ xmlXIncludeLoadDoc(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
     if (uri == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", url);
+	ctxt->nbErrors++;
 	return(-1);
     }
     if (uri->fragment != NULL) {
@@ -956,6 +1179,7 @@ xmlXIncludeLoadDoc(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
     if (URL == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", url);
+	ctxt->nbErrors++;
 	if (fragment != NULL)
 	    xmlFree(fragment);
 	return(-1);
@@ -965,7 +1189,8 @@ xmlXIncludeLoadDoc(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
      * Handling of references to the local document are done
      * directly through ctxt->doc.
      */
-    if ((URL[0] == 0) || (URL[0] == '#')) {
+    if ((URL[0] == 0) || (URL[0] == '#') ||
+	((ctxt->doc != NULL) && (xmlStrEqual(URL, ctxt->doc->URL)))) {
 	doc = NULL;
         goto loaded;
     }
@@ -1051,6 +1276,7 @@ loaded:
 	if (xptrctxt == NULL) {
 	    xmlGenericError(xmlGenericErrorContext,
 			"XInclude: could create XPointer context\n");
+	    ctxt->nbErrors++;
 	    xmlFree(URL);
 	    xmlFree(fragment);
 	    return(-1);
@@ -1060,6 +1286,7 @@ loaded:
 	    xmlGenericError(xmlGenericErrorContext,
 			"XInclude: XPointer evaluation failed: #%s\n",
 			fragment);
+	    ctxt->nbErrors++;
 	    xmlXPathFreeContext(xptrctxt);
 	    xmlFree(URL);
 	    xmlFree(fragment);
@@ -1076,6 +1303,7 @@ loaded:
 		xmlGenericError(xmlGenericErrorContext,
 			"XInclude: XPointer is not a range: #%s\n",
 			        fragment);
+		ctxt->nbErrors++;
 		xmlXPathFreeContext(xptrctxt);
 		xmlFree(URL);
 		xmlFree(fragment);
@@ -1108,12 +1336,14 @@ loaded:
 			xmlGenericError(xmlGenericErrorContext,
 			"XInclude: XPointer selects an attribute: #%s\n",
 					fragment);
+			ctxt->nbErrors++;
 			set->nodeTab[i] = NULL;
 			continue;
 		    case XML_NAMESPACE_DECL:
 			xmlGenericError(xmlGenericErrorContext,
 			"XInclude: XPointer selects a namespace: #%s\n",
 					fragment);
+			ctxt->nbErrors++;
 			set->nodeTab[i] = NULL;
 			continue;
 		    case XML_DOCUMENT_TYPE_NODE:
@@ -1128,15 +1358,21 @@ loaded:
 			xmlGenericError(xmlGenericErrorContext,
 			"XInclude: XPointer selects unexpected nodes: #%s\n",
 					fragment);
+			ctxt->nbErrors++;
 			set->nodeTab[i] = NULL;
 			set->nodeTab[i] = NULL;
 			continue; /* for */
 		}
 	    }
 	}
-	ctxt->incTab[nr]->inc =
-	    xmlXIncludeCopyXPointer(ctxt, ctxt->doc, doc, xptr);
-	xmlXPathFreeObject(xptr);
+	if (doc == NULL) {
+	    ctxt->incTab[nr]->xptr = xptr;
+	    ctxt->incTab[nr]->inc = NULL;
+	} else {
+	    ctxt->incTab[nr]->inc =
+		xmlXIncludeCopyXPointer(ctxt, ctxt->doc, doc, xptr);
+	    xmlXPathFreeObject(xptr);
+	}
 	xmlXPathFreeContext(xptrctxt);
 	xmlFree(fragment);
     }
@@ -1193,12 +1429,14 @@ xmlXIncludeLoadTxt(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
     if (uri == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", url);
+	ctxt->nbErrors++;
 	return(-1);
     }
     if (uri->fragment != NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		"XInclude: fragment identifier forbidden for text: %s\n",
 		uri->fragment);
+	ctxt->nbErrors++;
 	xmlFreeURI(uri);
 	return(-1);
     }
@@ -1207,6 +1445,7 @@ xmlXIncludeLoadTxt(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
     if (URL == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", url);
+	ctxt->nbErrors++;
 	return(-1);
     }
 
@@ -1217,6 +1456,7 @@ xmlXIncludeLoadTxt(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
     if (URL[0] == 0) {
 	xmlGenericError(xmlGenericErrorContext,
 		"XInclude: text serialization of document not available\n");
+	ctxt->nbErrors++;
 	xmlFree(URL);
 	return(-1);
     }
@@ -1247,6 +1487,7 @@ xmlXIncludeLoadTxt(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
 	if (enc == XML_CHAR_ENCODING_ERROR) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: encoding %s not supported\n", encoding);
+	    ctxt->nbErrors++;
 	    xmlFree(encoding);
 	    xmlFree(URL);
 	    return(-1);
@@ -1281,6 +1522,7 @@ xmlXIncludeLoadTxt(xmlXIncludeCtxtPtr ctxt, const xmlChar *url, int nr) {
 	    if (!IS_CHAR(cur)) {
 		xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: %s contains invalid char %d\n", URL, cur);
+		ctxt->nbErrors++;
 	    } else {
 		xmlNodeAddContentLen(node, &content[i], l);
 	    }
@@ -1437,6 +1679,7 @@ xmlXIncludePreloadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
 	xmlFree(base);
     if (URI == NULL) {
 	xmlGenericError(xmlGenericErrorContext, "XInclude: failed build URL\n");
+	ctxt->nbErrors++;
 	return(-1);
     }
 
@@ -1447,6 +1690,7 @@ xmlXIncludePreloadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     if (uri == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", URI);
+	ctxt->nbErrors++;
 	xmlFree(URI);
 	return(-1);
     }
@@ -1459,6 +1703,7 @@ xmlXIncludePreloadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     if (URL == NULL) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value URI %s\n", URI);
+	ctxt->nbErrors++;
 	if (fragment != NULL)
 	    xmlFree(fragment);
 	xmlFree(URI);
@@ -1517,6 +1762,7 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
 	href = xmlGetProp(cur, XINCLUDE_HREF);
 	if (href == NULL) {
 	    xmlGenericError(xmlGenericErrorContext, "XInclude: no href\n");
+	    ctxt->nbErrors++;
 	    return(-1);
 	}
     }
@@ -1533,6 +1779,7 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
 	    xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: invalid value %s for %s\n",
 		            parse, XINCLUDE_PARSE);
+	    ctxt->nbErrors++;
 	    if (href != NULL)
 		xmlFree(href);
 	    if (parse != NULL)
@@ -1566,6 +1813,7 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     }
     if (URI == NULL) {
 	xmlGenericError(xmlGenericErrorContext, "XInclude: failed build URL\n");
+	ctxt->nbErrors++;
 	if (parse != NULL)
 	    xmlFree(parse);
 	if (href != NULL)
@@ -1615,6 +1863,7 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
 	xmlGenericError(xmlGenericErrorContext,
 		    "XInclude: could not load %s, and no fallback was found\n",
 		        URI);
+	ctxt->nbErrors++;
     }
 
     /*
@@ -1642,7 +1891,7 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
  */
 static int
 xmlXIncludeIncludeNode(xmlXIncludeCtxtPtr ctxt, int nr) {
-    xmlNodePtr cur, end, list;
+    xmlNodePtr cur, end, list, tmp;
 
     if (ctxt == NULL)
 	return(-1);
@@ -1652,6 +1901,41 @@ xmlXIncludeIncludeNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     if (cur == NULL)
 	return(-1);
 
+    /*
+     * If we stored an XPointer a late computation may be needed
+     */
+    if ((ctxt->incTab[nr]->inc == NULL) &&
+	(ctxt->incTab[nr]->xptr != NULL)) {
+	ctxt->incTab[nr]->inc =
+	    xmlXIncludeCopyXPointer(ctxt, ctxt->doc, ctxt->doc,
+		                    ctxt->incTab[nr]->xptr);
+	xmlXPathFreeObject(ctxt->incTab[nr]->xptr);
+	ctxt->incTab[nr]->xptr = NULL;
+    }
+    list = ctxt->incTab[nr]->inc;
+    ctxt->incTab[nr]->inc = NULL;
+
+    /*
+     * Check against the risk of generating a multi-rooted document
+     */
+    if ((cur->parent != NULL) &&
+	(cur->parent->type != XML_ELEMENT_NODE)) {
+	int nb_elem = 0;
+
+	tmp = list;
+	while (tmp != NULL) {
+	    if (tmp->type == XML_ELEMENT_NODE)
+		nb_elem++;
+	    tmp = tmp->next;
+	}
+	if (nb_elem > 1) {
+	    xmlGenericError(xmlGenericErrorContext, 
+		    "XInclude error: would result in multiple root nodes\n");
+	    ctxt->nbErrors++;
+	    return(-1);
+	}
+    }
+
     /*
      * Change the current node as an XInclude start one, and add an
      * entity end one
@@ -1661,6 +1945,7 @@ xmlXIncludeIncludeNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     if (end == NULL) {
 	xmlGenericError(xmlGenericErrorContext, 
 		"XInclude: failed to build node\n");
+	ctxt->nbErrors++;
 	return(-1);
     }
     end->type = XML_XINCLUDE_END;
@@ -1669,8 +1954,6 @@ xmlXIncludeIncludeNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     /*
      * Add the list of nodes
      */
-    list = ctxt->incTab[nr]->inc;
-    ctxt->incTab[nr]->inc = NULL;
     while (list != NULL) {
 	cur = list;
 	list = list->next;
@@ -1684,6 +1967,7 @@ xmlXIncludeIncludeNode(xmlXIncludeCtxtPtr ctxt, int nr) {
 
 /**
  * xmlXIncludeTestNode:
+ * @ctxt: the XInclude processing context
  * @node: an XInclude node
  *
  * test if the node is an XInclude node
@@ -1691,19 +1975,61 @@ xmlXIncludeIncludeNode(xmlXIncludeCtxtPtr ctxt, int nr) {
  * Returns 1 true, 0 otherwise
  */
 static int
-xmlXIncludeTestNode(xmlNodePtr node) {
+xmlXIncludeTestNode(xmlXIncludeCtxtPtr ctxt, xmlNodePtr node) {
     if (node == NULL)
 	return(0);
     if (node->ns == NULL)
 	return(0);
-    if ((xmlStrEqual(node->name, XINCLUDE_NODE)) &&
-	(xmlStrEqual(node->ns->href, XINCLUDE_NS))) return(1);
+    if (xmlStrEqual(node->ns->href, XINCLUDE_NS)) {
+	if (xmlStrEqual(node->name, XINCLUDE_NODE)) {
+	    xmlNodePtr child = node->children;
+	    int nb_fallback = 0;
+
+	    while (child != NULL) {
+		if ((child->type == XML_ELEMENT_NODE) &&
+		    (child->ns != NULL) &&
+		    (xmlStrEqual(child->ns->href, XINCLUDE_NS))) {
+		    if (xmlStrEqual(child->name, XINCLUDE_NODE)) {
+			xmlGenericError(xmlGenericErrorContext,
+			    "XInclude: %s has an %s child\n",
+					XINCLUDE_NODE, XINCLUDE_NODE);
+			ctxt->nbErrors++;
+			return(0);
+		    }
+		    if (xmlStrEqual(child->name, XINCLUDE_FALLBACK)) {
+			nb_fallback++;
+		    }
+		}
+		child = child->next;
+	    }
+	    if (nb_fallback > 1) {
+		xmlGenericError(xmlGenericErrorContext,
+		    "XInclude: %s has %d %s children\n",
+				XINCLUDE_NODE, nb_fallback, XINCLUDE_FALLBACK);
+		ctxt->nbErrors++;
+		return(0);
+	    }
+	    return(1);
+	}
+	if (xmlStrEqual(node->name, XINCLUDE_FALLBACK)) {
+	    if ((node->parent == NULL) ||
+		(node->parent->type != XML_ELEMENT_NODE) ||
+		(node->parent->ns == NULL) ||
+		(!xmlStrEqual(node->parent->ns->href, XINCLUDE_NS)) ||
+		(!xmlStrEqual(node->parent->name, XINCLUDE_NODE))) {
+		xmlGenericError(xmlGenericErrorContext,
+		    "XInclude: %s is not the child of an %s\n",
+			        XINCLUDE_FALLBACK, XINCLUDE_NODE);
+		ctxt->nbErrors++;
+	    }
+	}
+    }
     return(0);
 }
 
 /**
  * xmlXIncludeDoProcess:
- * @ctxt: 
+ * @ctxt: the XInclude processing context
  * @doc: an XML document
  *
  * Implement the XInclude substitution on the XML document @doc
@@ -1722,22 +2048,28 @@ xmlXIncludeDoProcess(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc) {
     if (ctxt == NULL)
 	return(-1);
 
+    if (doc->URL != NULL) {
+	ret = xmlXIncludeURLPush(ctxt, doc->URL);
+	if (ret < 0)
+	    return(-1);
+    }
+
     /*
      * First phase: lookup the elements in the document
      */
     cur = xmlDocGetRootElement(doc);
-    if (xmlXIncludeTestNode(cur))
+    if (xmlXIncludeTestNode(ctxt, cur) == 1)
 	xmlXIncludePreProcessNode(ctxt, cur);
     while (cur != NULL) {
 	/* TODO: need to work on entities -> stack */
 	if ((cur->children != NULL) &&
 	    (cur->children->type != XML_ENTITY_DECL)) {
 	    cur = cur->children;
-	    if (xmlXIncludeTestNode(cur))
+	    if (xmlXIncludeTestNode(ctxt, cur))
 		xmlXIncludePreProcessNode(ctxt, cur);
 	} else if (cur->next != NULL) {
 	    cur = cur->next;
-	    if (xmlXIncludeTestNode(cur))
+	    if (xmlXIncludeTestNode(ctxt, cur))
 		xmlXIncludePreProcessNode(ctxt, cur);
 	} else {
 	    do {
@@ -1745,7 +2077,7 @@ xmlXIncludeDoProcess(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc) {
 		if (cur == NULL) break; /* do */
 		if (cur->next != NULL) {
 		    cur = cur->next;
-		    if (xmlXIncludeTestNode(cur))
+		    if (xmlXIncludeTestNode(ctxt, cur))
 			xmlXIncludePreProcessNode(ctxt, cur);
 		    break; /* do */
 		}
@@ -1764,10 +2096,14 @@ xmlXIncludeDoProcess(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc) {
     /*
      * Third phase: extend the original document infoset.
      */
-    for (i = ctxt->incBase;i < ctxt->incNr; i++) {
-	xmlXIncludeIncludeNode(ctxt, i);
+    if (ctxt->nbErrors == 0) {
+	for (i = ctxt->incBase;i < ctxt->incNr; i++) {
+	    xmlXIncludeIncludeNode(ctxt, i);
+	}
     }
 
+    if (doc->URL != NULL)
+	xmlXIncludeURLPop(ctxt);
     return(ret);
 }
 
@@ -1791,6 +2127,8 @@ xmlXIncludeProcess(xmlDocPtr doc) {
     if (ctxt == NULL)
 	return(-1);
     ret = xmlXIncludeDoProcess(ctxt, doc);
+    if ((ret >= 0) && (ctxt->nbErrors > 0))
+	ret = -1;
 
     xmlXIncludeFreeContext(ctxt);
     return(ret);
