@@ -1,14 +1,20 @@
 /*
  * xpointer.c : Code to handle XML Pointer
  *
- * World Wide Web Consortium Working Draft 03-March-1998 
+ * Base implementation was made accordingly to
+ * W3C Candidate Recommendation 7 June 2000
  * http://www.w3.org/TR/2000/CR-xptr-20000607
  *
+ * Added support for the element() scheme described in:
+ * W3C Proposed Recommendation 13 November 2002
+ * http://www.w3.org/TR/2002/PR-xptr-element-20021113/  
+ *
  * See Copyright for the status of this software.
  *
  * daniel@veillard.com
  */
 
+#define IN_LIBXML
 #include "libxml.h"
 
 /*
@@ -814,6 +820,8 @@ xmlXPtrWrapLocationSet(xmlLocationSetPtr val) {
  *									*
  ************************************************************************/
 
+static void xmlXPtrEvalChildSeq(xmlXPathParserContextPtr ctxt, xmlChar *name);
+
 /*
  * Macros for accessing the content. Those should be used only by the parser,
  * and not exported.
@@ -977,6 +985,24 @@ xmlXPtrEvalXPtrPart(xmlXPathParserContextPtr ctxt, xmlChar *name) {
 	CUR_PTR = buffer;
 	xmlXPathEvalExpr(ctxt);
 	CUR_PTR=left;
+    } else if (xmlStrEqual(name, (xmlChar *) "element")) {
+	const xmlChar *left = CUR_PTR;
+	xmlChar *name2;
+
+	CUR_PTR = buffer;
+	if (buffer[0] == '/') {
+	    xmlXPathRoot(ctxt);
+	    xmlXPtrEvalChildSeq(ctxt, NULL);
+	} else {
+	    name2 = xmlXPathParseName(ctxt);
+	    if (name2 == NULL) {
+		CUR_PTR = left;
+		xmlFree(buffer);
+		XP_ERROR(XPATH_EXPR_ERROR);
+	    }
+	    xmlXPtrEvalChildSeq(ctxt, name2);
+	}
+	CUR_PTR = left;
 #ifdef XPTR_XMLNS_SCHEME
     } else if (xmlStrEqual(name, (xmlChar *) "xmlns")) {
 	const xmlChar *left = CUR_PTR;
@@ -1020,6 +1046,8 @@ xmlXPtrEvalXPtrPart(xmlXPathParserContextPtr ctxt, xmlChar *name) {
 	
 	xmlXPathRegisterNs(ctxt->context, prefix, URI);
 	CUR_PTR = left;
+	xmlFree(URI);
+	xmlFree(prefix);
 #endif /* XPTR_XMLNS_SCHEME */
     } else {
         xmlGenericError(xmlGenericErrorContext,
@@ -1628,11 +1656,7 @@ xmlXPtrNbLocChildren(xmlNodePtr node) {
         case XML_TEXT_NODE:
         case XML_CDATA_SECTION_NODE:
         case XML_ENTITY_REF_NODE:
-#ifndef XML_USE_BUFFER_CONTENT
 	    ret = xmlStrlen(node->content);
-#else
-	    ret = xmlBufferLength(node->content);
-#endif
 	    break;
 	default:
 	    return(-1);
@@ -2007,13 +2031,8 @@ xmlXPtrInsideRange(xmlXPathParserContextPtr ctxt, xmlXPathObjectPtr loc) {
 		    if (node->content == NULL) {
 			return(xmlXPtrNewRange(node, 0, node, 0));
 		    } else {
-#ifndef XML_USE_BUFFER_CONTENT
 			return(xmlXPtrNewRange(node, 0, node,
 					       xmlStrlen(node->content)));
-#else
-			return(xmlXPtrNewRange(node, 0, node,
-					       xmlBufferLength(node->content)));
-#endif
 		    }
 		}
 		case XML_ATTRIBUTE_NODE:
@@ -2044,13 +2063,8 @@ xmlXPtrInsideRange(xmlXPathParserContextPtr ctxt, xmlXPathObjectPtr loc) {
 			if (node->content == NULL) {
 			    return(xmlXPtrNewRange(node, 0, node, 0));
 			} else {
-#ifndef XML_USE_BUFFER_CONTENT
 			    return(xmlXPtrNewRange(node, 0, node,
 						   xmlStrlen(node->content)));
-#else
-			    return(xmlXPtrNewRange(node, 0, node,
-					       xmlBufferLength(node->content)));
-#endif
 			}
 		    }
 		    case XML_ATTRIBUTE_NODE:
@@ -2319,11 +2333,7 @@ xmlXPtrAdvanceChar(xmlNodePtr *node, int *indx, int bytes) {
 	len = 0;
 	if ((cur->type != XML_ELEMENT_NODE) &&
             (cur->content != NULL)) {
-#ifndef XML_USE_BUFFER_CONTENT
 	    len = xmlStrlen(cur->content);
-#else
-	    len = xmlBufferLength(cur->content);
-#endif
 	}
 	if (pos > len) {
 	    /* Strange, the indx in the text node is greater than it's len */
@@ -2386,18 +2396,9 @@ xmlXPtrMatchString(const xmlChar *string, xmlNodePtr start, int startindex,
 	    return(0);
 
 	if ((cur->type != XML_ELEMENT_NODE) && (cur->content != NULL)) {
-#ifndef XML_USE_BUFFER_CONTENT
 	    len = xmlStrlen(cur->content);
-#else
-	    len = xmlBufferLength(cur->content);
-#endif
 	    if (len >= pos + stringlen) {
-#ifndef XML_USE_BUFFER_CONTENT
 		match = (!xmlStrncmp(&cur->content[pos], string, stringlen));
-#else
-		len = (!xmlStrncmp(&xmlBufferContent(cur->content)[pos],
-			           string, stringlen));
-#endif
 		if (match) {
 #ifdef DEBUG_RANGES
 		    xmlGenericError(xmlGenericErrorContext,
@@ -2414,12 +2415,7 @@ xmlXPtrMatchString(const xmlChar *string, xmlNodePtr start, int startindex,
 		}
 	    } else {
                 int sub = len - pos;
-#ifndef XML_USE_BUFFER_CONTENT
 		match = (!xmlStrncmp(&cur->content[pos], string, sub));
-#else
-		len = (!xmlStrncmp(&xmlBufferContent(cur->content)[pos],
-			           string, sub));
-#endif
 		if (match) {
 #ifdef DEBUG_RANGES
 		    xmlGenericError(xmlGenericErrorContext,
@@ -2482,19 +2478,10 @@ xmlXPtrSearchString(const xmlChar *string, xmlNodePtr *start, int *startindex,
 
     while (cur != NULL) {
 	if ((cur->type != XML_ELEMENT_NODE) && (cur->content != NULL)) {
-#ifndef XML_USE_BUFFER_CONTENT
 	    len = xmlStrlen(cur->content);
-#else
-	    len = xmlBufferLength(cur->content);
-#endif
 	    while (pos <= len) {
 		if (first != 0) {
-#ifndef XML_USE_BUFFER_CONTENT
 		    str = xmlStrchr(&cur->content[pos], first);
-#else
-		    str = xmlStrchr(&xmlBufferContent(cur->content)[pos],
-			            first);
-#endif
 		    if (str != NULL) {
 			pos = (str - (xmlChar *)(cur->content));
 #ifdef DEBUG_RANGES
@@ -2580,11 +2567,7 @@ xmlXPtrGetLastChar(xmlNodePtr *node, int *indx) {
 	    cur = cur->last;
 	else if ((cur->type != XML_ELEMENT_NODE) &&
 	         (cur->content != NULL)) {
-#ifndef XML_USE_BUFFER_CONTENT
 	    len = xmlStrlen(cur->content);
-#else
-	    len = xmlBufferLength(cur->content);
-#endif
 	    break;
 	} else {
 	    return(-1);
